using #emfUtilities;

////////////////////////////////////////////////////////////////////////////////////////////



function eClassifier (ecoreobj) {
	local FuncMap = ecoreobj.funcMap;
	local mainPackage = ecoreobj.mainPackage;
	local funcArgs = [ @funcMap : FuncMap, @mainPackage : mainPackage ];
	ecoreobj = ecoreobj.xmlobj;
	//
	
	local ast		= nil;
	//local args 		= nil;
	//
	local classifier 			= ecoreobj;
	local classifierName	 	= classifier["$Attributes"]["name"];
	local classifierType	 	= classifier["$Attributes"]["xsi:type"];
	local classifierInherits	= classifier["$Attributes"]["eSuperTypes"];
	// Attributes of class
	if (local eStructuralFeatures = classifier["eStructuralFeatures"]) {
		local eStructuralFeaturesLength = std::tablength(eStructuralFeatures);
		for (local i = 0; i < eStructuralFeaturesLength; ++i) {
			local eStructuralFeature = eStructuralFeatures[i];
			local type = eStructuralFeature["$Attributes"]["xsi:type"];
			if (type != "ecore:EAttribute" and type != "ecore:EReference")
				std::error("Invalid structural feature, not implemented!");
			//
			funcArgs.xmlobj = eStructuralFeature;
			
			local childData = FuncMap [type] (funcArgs);
			
			//args = << ~args, ~(emfUtilities::expr_new(childData.arg)) >>;
			ast = << ~ast, ~(childData.ast) >>;
		}
	}
	// Methods of class
	if (local eOperations = classifier["eOperations"]) {
		local eOperationsLength = std::tablength(eOperations);
		for (local i = 0; i < eOperationsLength; ++i) {
			funcArgs.xmlobj = eOperations[i];
			
			local childData = FuncMap ["eOperation"] (funcArgs);			
			ast = << ~ast, ~(childData.ast) >>;
		}
	}
	
	// attributes and methods of class
	ast = << [ ~ast ] >>;
	
	// inheritance	
	if (not classifierInherits)
		ast = << return ~ast; >>;
	else {
		local baseClasses = std::strtokenise (classifierInherits, " ");
		
		local derivedObjName = "derived_" + std::strlower(classifierName);
		ast = <<
				// derived
				local ~(emfUtilities::id_new(derivedObjName)) = ~ast;
		>>;
		foreach (base, baseClasses) {
			local path = std::strtokenise (base, "#//");
			
			local baseClassCtor = << ~(emfUtilities::expr_new(mainPackage))()>>;
			local baseObjName = "base_" + std::strlower(mainPackage);		
			foreach(local name, path) {
				baseClassCtor = << ~baseClassCtor[~name]() >>;
				baseObjName += "_" + std::strlower(name);
			}
			
			ast = <<
				~ast;
				// base
				local ~(emfUtilities::id_new(baseObjName)) = ~baseClassCtor;
				//
				std::inherit (~(emfUtilities::expr_new(derivedObjName)), ~(emfUtilities::expr_new(baseObjName)));
			>>;
		}
		ast = <<
				~ast;
				//
				return ~(emfUtilities::expr_new(derivedObjName));
		>>;
		
	}
	// funtion declaration
	ast = << function ~(emfUtilities::id_new(classifierName)) ( /*~args*/ ) { ~ast; } >>;
	
	local data = [ @ast : ast, @name : classifierName ];
	return data;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// attributes and references ////////////////////////////////////////////////////////////////////

function eAttribute (ecoreobj) {
	local ast = nil;
	local attribute = ecoreobj.xmlobj;
	local name = attribute["$Attributes"].name;
	emfUtilities::CheckValidName(name);
	
	local idname = emfUtilities::id_new(name);
	local exprname = emfUtilities::expr_new(name);
	
	//
	local type = attribute["$Attributes"].eType;
	local value = attribute["$Attributes"].defaultValueLiteral;
	
	local checkTypeAst = nil;
	if (type != nil) {
		local DTypeAst = emfUtilities::GetDeltaTypeAst (type);
		local DType = emfUtilities::GetDeltaType (type);
		if (DTypeAst)
			checkTypeAst = <<
					if (std::typeof(~exprname) != ~DTypeAst)
						std::error (~("Error: invalid type ")+std::typeof(~exprname)+~(", "+name+" is type of "+DType+"!"));
			>>;
	}
	
	ast = << @~idname {
					@set (method (~exprname) {
								~checkTypeAst;
								@~idname = ~exprname; 
						  }
					)
					@get (method () { return @~idname; })
			  }
	>>;

	//local initialization = (value ?
	//	<<@~idname : (~exprname ? ~(emfUtilities::expr_new(value)) : ~value )>> : 
	//	<<@~idname : (~exprname)>>);
	local val = emfUtilities::GetValueByType(value, type);
	local initialization = (std::isnil(val) ? nil : <<@~idname : ~val>>);
	ast = << ~ast, ~initialization >>;

	//
	//return data
	local data = [@ast : ast/*,  @arg : name*/];
	return data;
}

function eReference (ecoreobj) {
	local ast = nil;
	local attribute = ecoreobj.xmlobj;
	local name = attribute["$Attributes"].name;
	emfUtilities::CheckValidName(name);
	
	local idname = emfUtilities::id_new(name);
	local exprname = emfUtilities::expr_new(name);	
	
	ast = << @~idname { @set (method(~exprname) { @~idname = ~exprname; }) @get (method() { return @~idname; }) } >>;
	
	//return data
	local data = [@ast : ast/*,  @arg : name*/];	
	return data;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////// operations and annotations(code) /////////////////////////////////////////////////////////////

function eOperation (ecoreobj) {
	local FuncMap = ecoreobj.funcMap;
	local funcArgs = [ @funcMap : FuncMap, @mainPackage : ecoreobj.mainPackage ];
	ecoreobj = ecoreobj.xmlobj;
	//
	
	local ast    	= nil;
	local params	= nil;
	local body		= nil;
	//
	local operation = ecoreobj;
	local operationName = operation["$Attributes"]["name"];
	//eParameters of method
	if (local eParameters = operation["eParameters"]) {
		local eParametersLength = std::tablength(eParameters);
		for (local i = 0; i < eParametersLength; ++i) {
			local name = eParameters[i]["$Attributes"]["name"];
			params = << ~params, ~(emfUtilities::expr_new(name)) >>;
		}
	}
	//eAnnotations of method
	if (local eAnnotations = operation["eAnnotations"]) {
		local eAnnotationsLength = std::tablength(eAnnotations);
		for (local i = 0; i < eAnnotationsLength; ++i) {
			funcArgs.xmlobj = eAnnotations[i];
			
			local childData = FuncMap ["eAnnotation"] (funcArgs);
			
			body = <<
					~body;
					~(childData.ast);
			>>;
		}
	}
	//
	ast = << method ~(emfUtilities::id_new(operationName)) ( ~params ) {  ~body; } >>;
	local data = [ @ast : ast ];
	return data;
}


function InvalidCode () {
	std::print ("Invalid source code not added!\n");
}

function eAnnotation (ecoreobj) {
	local ast = nil;
	local annotation = ecoreobj.xmlobj;
	//anotation attribute - maybe use it later, now in comments
	//source = annotation["source"];
	
	//details - code
	if (local details = annotation.details) {
		local detailsLength = std::tablength(details);
		for (local i = 0; i < detailsLength; ++i) {
			local detail = details[i];
			local key = detail["$Attributes"]["key"];
			local strBodyCode = detail["$Attributes"]["value"];
			
			local programast = std::vmparsestring(strBodyCode, ::InvalidCode);
			local stmtsast = std::ast_get_child(programast, 0);
			//
			ast = <<
				~ast;
				~stmtsast;
			>>;
			//TODO ask john: stmtsast is copy, need original?
			ast [key] = stmtsast;
		}
	}
	local data = [ @ast : ast ];
	return data;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////

function eDataType (ecoreobj) {
	// In a typed setting, this would make sense for example to import a type.
	// Here it makes no sense. Even arguments of eDataType receive their value as
	// constructor arguments, so there is no need to invoke any external function.
	/*
	local FuncMap = ecoreobj.funcMap;
	local dataType = ecoreobj.xmlobj;
	//
	local funcArgs = [ @funcMap : FuncMap ];
	
	local ast = nil;
	//
	local dataTypeName = dataType["$Attributes"]["name"];
	//
	local dataTypeInstanceTypeName = dataType["$Attributes"]["instanceTypeName"];
	
	//TODO: complete ast code...
	*/
	local data = [/*@ast : nil*/];
	return data;
}

function eEnum (ecoreobj) {
	local FuncMap = ecoreobj.funcMap;
	local funcArgs = [ @funcMap : FuncMap, @mainPackage : ecoreobj.mainPackage ];
	ecoreobj = ecoreobj.xmlobj;
	//
	
	local ast = nil;
	local constsAst = nil;
	//
	local enum = ecoreobj;
	local enumName = enum["$Attributes"]["name"];
	//
	if (local eLiterals = enum["eLiterals"]) {
		local eLiteralsLength = std::tablength(eLiterals);
		for (local i = 0; i < eLiteralsLength; ++i) {
			local eLiteral = eLiterals[i];
			//
			local name = eLiteral["$Attributes"]["name"];
			local value = eLiteral["$Attributes"]["value"];
			if (not value)
				value = 0;
			else
				value = (std::strtonum(value)==0 ? value : std::strtonum(value));
			
			constsAst = <<
						~constsAst;
						const ~(emfUtilities::id_new(name)) = ~value;
			>>;
			ast = <<
					~ast,
					{ ~name : ~(emfUtilities::expr_new(name)) }
			>>;
		}
	}
	
	ast = <<
			function ~(emfUtilities::id_new(enumName)) {
				~constsAst;
				return [
					~ast
				];
			}
	>>;
	
	local data = [ @ast : ast, @name : enumName ];
	return data;
}
