using std;

function Singleton() {
	function ListContains(list, target) {
		foreach(local item, list)
			if (item == target)
				return true;
		return false;	
	}
	function topLevelItems(ast, type) {
		local nesting = 0;
		local items = std::list_new();
		local visitor = std::astvisitor_new();
		visitor.set_handler(type, function(node, id, entering){
			if (entering) {
				if (++nesting == 1)
					items.push_back(node);
			}
			else
				--nesting;
		});
		visitor.set_handler("NewAttribute", function(node, id, entering){
			if (entering) ++nesting; else --nesting;
		});
		ast.accept_preorder(visitor);
		assert nesting == 0;
		return items;
	}
	function getFuncName(func) {
		assert func.get_tag() == "Function";
		return func.get_child("name").get_attribute("name");
	}

	if (isundefined(static decorator))
		decorator = [
			@packages : list_new(),
			@enums : list_new(),
			
			method AddPackage(package) { @packages.push_back(package); },
			method AddEnum(enum) { @enums.push_back(enum); },
			method IsSubPackage(path) { return ListContains(@packages, path); },
			method IsEnum(path) { return ListContains(@enums, path); },
			
			method decorateEnum(ast) {
				assert ast.get_tag() == "Function";
				local defs = topLevelItems(ast.get_child("body"), "ConstDef");
				foreach(local def, defs)
					ast[def.get_child("name").get_attribute("name")] = def;
			},
			method decorateOperation(ast) {
				assert ast.get_tag() == "Function";
				ast.name = getFuncName(ast);
				ast.formals = ast.get_child("formals");
				ast.body = ast.get_child("body");
			},
			method decorateClass(ast) {
				assert ast.get_tag() == "Function";
				//decorate operations
				local funcs = topLevelItems(ast.get_child("body"), "Function");
				foreach(local func, funcs) {
					@decorateOperation(func);
					ast[func.name] = func;
				}
				local ctors = topLevelItems(ast.get_child("body"), "TableConstructor");
				assert ctors.total() == 1;
				ast["@ctor"] = ctors.front();
				//TODO: should we decorate attributes or references?
				ast["@name"] = getFuncName(ast);
			},
			method decoratePackage(ast, path) {
				assert ast.get_tag() == "Function";
				local packageName = getFuncName(ast);
				path += packageName + "/";
				local funcs = topLevelItems(ast.get_child("body"), "Function");
				foreach(local func, funcs) {
					local name = getFuncName(func);		
					if (@IsSubPackage(path + name))
						@decoratePackage(func, path);
					else if (@IsEnum(path + name))
						@decorateEnum(func);
					else
						@decorateClass(func);
					ast[name] = func;
				}
				ast["@name"] = packageName;
			},
			method decorateTopLevelCode(ast, ecoreobj) {
				@generateModelInfo(ecoreobj, "");
				local funcs = topLevelItems(ast, "Function");
				assert funcs.total() == 1;
				local func = funcs.front();
				@decoratePackage(func, "");
				ast[ func["@name"] ] = func;
			},
			method generateModelInfo(ecoreobj, path) {
				local name = ecoreobj["$Attributes"]["name"];
				@AddPackage(path + name);
				path += name + "/";
				if (local eClassifiesObj = ecoreobj["eClassifiers"]) {
					local obj = eClassifiesObj;
					local n = std::tablength(obj);
					for (local i = 0; i < n; ++i)
						 if (obj[i]["$Attributes"]["xsi:type"] == "ecore:EEnum")
							@AddEnum(path + obj[i]["$Attributes"]["name"]);
				}
				if (local eSubpackagesObj = ecoreobj["eSubpackages"]) {
					local obj = eSubpackagesObj;
					local n = std::tablength(obj);
					for (local i = 0; i < n; ++i)
						@generateModelInfo(obj[i], path);
				}
			}
		];
	return decorator;
}