using #emfUtilities;

function ePackage (ecoreobj, topLevel) {
	local main = (arguments.total() > 1 ? topLevel : false);
	
	local FuncMap = ecoreobj.funcMap;
	local funcArgs = [ @funcMap : FuncMap ];
	if (not main)
		funcArgs.mainPackage = ecoreobj.mainPackage;
	ecoreobj = ecoreobj.xmlobj;
	
	local ast = nil;
	
	local name = nil;
	if (local attributes = ecoreobj["$Attributes"])
		name = attributes["name"];
	emfUtilities::CheckValidName(name);
	if (main)
		funcArgs.mainPackage = name;
		
	//data for classifiers-subpackages
	
	local eClassifiesData = [];	//empty ast
	if (local eClassifiesObj = ecoreobj["eClassifiers"]) {
		funcArgs.xmlobj	= eClassifiesObj;
		eClassifiesData	= FuncMap["eClassifiers"] (funcArgs);
	}
	//
	local eSubpackagesData = [];	//empty ast
	if (local eSubpackagesObj = ecoreobj["eSubpackages"]) {
		funcArgs.xmlobj	 = eSubpackagesObj;
		eSubpackagesData = FuncMap["eSubpackages"] (funcArgs);
	}
	//
	local returnObjList = << ~(eClassifiesData.returnObjList), ~(eSubpackagesData.returnObjList) >>;
	ast = <<
			function ~(emfUtilities::id_new(name)) () {
				~(eClassifiesData.ast);
				~(eSubpackagesData.ast);
				return [
					~returnObjList
				];
			}
	>>;
	
	local data = [ @ast : ast, @name : name ];
	return data;
}

function eSubpackages (ecoreobj) {
	local FuncMap = ecoreobj.funcMap;
	local funcArgs = [ @funcMap : FuncMap, @mainPackage : ecoreobj.mainPackage ];
	ecoreobj = ecoreobj.xmlobj;
	//
	local ast = nil;
	local returnObjList = nil;
	
	local subPackagesLength = std::tablength(ecoreobj);	
	for (local i = 0; i < subPackagesLength; ++i) {
		local subPackage = ecoreobj[i];
		
		// call children to get their asts
		funcArgs.xmlobj = subPackage;
		local childData = FuncMap["Package"] ( funcArgs );
		
		returnObjList = << ~returnObjList, @~(emfUtilities::id_new(childData.name)) : ~(emfUtilities::expr_new(childData.name)) >>;
		
		ast = <<
			~ast;
			~(childData.ast);
		>>;
	}
	
	local data = [ @ast : ast, @returnObjList : returnObjList ];
	return data;
}

function eClassifiers (ecoreobj) {
	local FuncMap = ecoreobj.funcMap;
	local funcArgs = [ @funcMap : FuncMap, @mainPackage : ecoreobj.mainPackage ];
	ecoreobj = ecoreobj.xmlobj;

	//
	local ast = nil;
	local returnObjList = nil;
	
	local classifiersLength = std::tablength(ecoreobj);
	for (local i = 0; i < classifiersLength; ++i) {
		local classifier = ecoreobj[i];
		local funcname = classifier["$Attributes"]["xsi:type"];
		// call children to get their asts
		funcArgs.xmlobj = classifier;
		local childData = FuncMap[funcname] ( funcArgs );
		
		//check for dataType that is emty return
		if (funcname!="ecore:EDataType") {
			returnObjList = << ~returnObjList, @~(emfUtilities::id_new(childData.name)) : ~(emfUtilities::expr_new(childData.name)) >>;
			ast = <<
				~ast;
				~(childData.ast);
			>>;
		}
	}
	local data = [ @ast : ast, @returnObjList : returnObjList ];
	
	return data;
}
