using #Mapping;
using #Utilities;
using #EventInserter;
using #Data;

const nl = "\n";

const begin_msg = "///////////////////////////////////////////////////////////////////////////
//
// DELTA GUI code generated XRC Decoder
//
///////////////////////////////////////////////////////////////////////////"+nl+nl+nl;

astid = 0;

dllXML = nil;
function Initialize () {
	dllXML = std::dllimportdeltalib(xml::DLL);
}

function CleanUp () {
	std::dllunimportdeltalib(dllXML);
}


/**
 *  Map widget names to corespodent ast function nodes
 *
 **/
function local AstMappingGuiNodes (guiobj) {
	// map of all gui var with functions nodes
	astmap = [];
	functions = std::list_new();
	//
	visitor = std::astvisitor_new();
	visitor.set_handler("Function", function(node, id, entering) { if (entering) functions.push_back(node); });
	guiobj.ast.accept_preorder(visitor);

	foreach (func, functions) {
		local astfunction = std::ast_get_child(func, "name");
		// get funcname
		funcname = std::ast_get_attribute(astfunction, "name");
		//
		foreach (key, std::tabindices(guiobj.astMap))
			if (guiobj.astMap [key][0] == funcname) {
				astmap [key] = func;
				break;
			}

	}
	
	return astmap;
}

function local MapType (guiobj) {
	map = [];
	foreach (name, std::tabindices(guiobj.astMap))
		map[name] = guiobj.astMap[name][1];
	
	return map;
}

function local xrc2astData (rootobj, registerWidgetFuncname) {
	funcMap = Mapping::wxObjectMapping();
	source  = begin_msg;
	astgui = nil;
	astchild = nil;
	source += "function GUI () {"+nl;
	Utilities::tabincrease();
	rootlength = std::tablength(rootobj);
	subobj = [ @funcMap : funcMap, @astMap : [] ];
	
	for (i=0; i<rootlength; ++i) {
		mainobj = rootobj[i];
		funcname = mainobj["$Attributes"]["class"];
		//
		subobj.xrcObj = mainobj;
		
		if (std::typeof(registerWidgetFuncname) != std::TYPEOF_BOOL)
			std::error ("Error: invalid call AstGuiGenerator input arg2: not bool type!");
		
		
		subobj.registerWidgetFuncName = registerWidgetFuncname;
		//
		astchild = funcMap [funcname] (subobj);
		source += astchild.src + nl;
		astgui = << ~astgui; ~(astchild.ast); >>;
		//
		source += Utilities::tab() + astchild.funcname + "();"+ nl;
		tmpast =
		<<
			local frame = ~(Utilities::expr_new(astchild.funcname)) (guiobj);
			frame.show();
		>>;
		astgui =
		<<
			~astgui;
			~tmpast;
		>>;
	}
	Utilities::tabdecrease();
	source += "}" + nl;
	astgui = <<
		function create_gui (guiobj) {
			~astgui;
		}
	>>;
	if (registerWidgetFuncname)
		astgui = <<
			function GUIWidgets() {
				if (not static widgets)
					widgets = [];
				return widgets;
			}
			~astgui;
		>>;
	
	createdGUI = [
		@ast	: astgui,
		@astMap	: astchild.astMap,
		@src	: source
	];
	return createdGUI;
}

/**
 *
 *	0) xrcpath		: string
 *  1) registerWidgetFuncName	: bool
 *	2) eventsObj	: [ /*event1*/["WidgetName", "EventType", "Funcname"], /*event2*/[...], ..... ]
 *	
 **/
function AstGuiGenerator (xrcpath, registerWidgetFuncname, eventsObj) {
	//
	xrcobj = xml::load(xrcpath);
	//
	local ast = nil;
	
	if (not xrcobj ) {
		std::print("Failed to load XRC file!", nl);
		std::print(xml::loadgeterror(), nl);
		return nil;
	}

//	try {
		rootobj = xrcobj["object"];
		// create ast
		local data		= ::xrc2astData (rootobj, (registerWidgetFuncname?registerWidgetFuncname:false));
		data.typesMap	= ::MapType (data);
		data.astMap		= ::AstMappingGuiNodes(data);
		// ast ident for Data Loader
		data.ast.id		= ::astid;
		++::astid;
		
		// add events in ast
		if (eventsObj)
			if (std::typeof(eventsObj)==std::TYPEOF_OBJECT)
				EventInserter::AddEventsAstGUI (data, eventsObj);
			else
				std::error("#arg1 is not valid type: " + std::typeof(eventsObj));
		ast = data.ast;
		Data::loadAst(ast);
//	}
	// cath if xrc file is invalid or not supported wxwidgets
//	trap e {
//		std::print(nl,">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>", nl, nl);
//		std::print("Invalid type of Input!", nl);
//		std::print("Details:", nl, e, nl);
//		std::print(nl, "<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<", nl, nl);
//		return -1;
//	}
	
	return ast;
}

