using #Utilities;



function GenerateInvalidChildObject (type) {
	return <<
			try {
				name = objectxrc["object"][index]["$Attributes"]["class"];
				std::error ("Error: "+~type+" must not include object childs."+nl+"Child object named as "+name+"!" +nl);
			}
			trap e {
				std::error ("Error: "+~type+" must not include object childs."+nl+"Object tag in xrc is invalid!" +nl);
			}
		>>;
}

function GenerateObjectName (id, type) {
	return <<
			++::~(id);
			local ~(id) = ::~(id);
			if (std::isnil(objectname)) {
				objectname = "g_" + ~(type) + ~(id);
			}
	>>;
}

function GenCheckSizersInvalidCase (sizertype) {
	return <<
		if (funcname != "sizeritem" and funcname != "spacer") {
			std::error("Error: Invalid object child for "+~sizertype+" in xrc file! Child name: "+funcname+"!");
		}
	>>;
}

function GenerateFontCase () {
	return
	<<
		if (child == "font") {
			objectast = <<
						~~(objectast); 
						~~(astchild.ast);
						local font = ~~(Utilities::expr_new(astchild.funcname)) (guichildobj);
						~~(Utilities::lvalue_new(objectname)) ["setfont"] ( font );
			>>;
			//
			continue;
		}
	>>;
}

function GenerateWindowMethodname (...) {
	code =
	<<
		local methodname = "set";
		//special cases
		if (child == "bg") {
			methodname += "backgroundcolour";
			child = "colour";
		}
		else
		if (child == "fg") {
			methodname += "foregroundcolour";
			child = "colour";
		}
		else
			~~code;
	>>;
	lastCode =
	<<
		if (child == "pos") {
			methodname = "move";
		}
		else
		if (child == "style") {
			methodname += "window" + child;
		}
		else
		if (child == "exstyle") {
			methodname += "extrastyle";
		}
		else
		if (child == "hidden") {
			methodname = "hide";
		}
		else
		if (child == "enabled") {
			methodname = "enable";
		}
		else {
			methodname += std::strlower (child);
		}
	>>;
	// inject intermidiate code if there is
	if (arguments[0] != nil) {
		code = code.inject (arguments[0]);
		code.decr_esc_cardinalities();
	}
	code = code.inject (lastCode);
	code.decr_esc_cardinalities();
	return code;
}

function GenFixObj2Parent () {
	return <<
		if (object.parentType == "wx_panel") {
			bodyast = <<
							~~bodyast;
							//guiobj["parent"]["layout"] ();
							~~( Utilities::expr_new(objectname) ) ["fit"] (guiobj["parent"]);
			>>;
		}
	>>;
}

function GenerateChildObjectCode (...) {
	return <<
		//
		objects = objectxrc["object"];
		//parser xml in delta not in order parse...
		//foreach (index, std::tabindices(objects)) {
		for (index=0; index<std::tablength(objects); ++index) {
			funcname = objects[index]["$Attributes"]["class"];
			// check for invalid funcname cases
			~(arguments[0]);
			// special case change funcname
			~(arguments[1]);
			//
			childobj.parent = objectname;
			childobj.xrcObj = objects[index];
			//
			astchild = funcmap [ funcname ] (childobj);
			objectast = <<
							~~objectast;
							~~(astchild.ast);
			>>;
			objectsrc += astchild.src + nl;
			
			//
			local callast = nil;
			
			
			// NO need to keep in local var the call of create child function
//			if ( ~(arguments[2])  ) {
				objectsrc += Utilities::tab() + astchild.funcname + "( " + objectname + " );" + nl;
				callast =
				<<
					~~(Utilities::expr_new(astchild.funcname)) (/*~~(Utilities::expr_new(objectname))*/ guichildobj);
				>>;
//			}
			// Need to keep in local var the call of create child function
//			else {
//				local varid = "";
//				for(i=7;i<std::strlen(astchild.funcname);++i) {
//					varid += std::strchar(astchild.funcname, i);
//				}
				//
//				objectsrc += Utilities::tab() + "local " + varid + " = " + astchild.funcname + "( " + objectname + " );" + nl;
///				callast   =
//				<<
//					local ~~(Utilities::id_new(varid))  = ~~(Utilities::expr_new(astchild.funcname)) (~~(Utilities::expr_new(objectname)));
//				>>;
//			}
			objectast  = <<
							~~objectast;
							~~callast;
			>>;
			~(arguments[2]);
		}
	>>;
}

function GenerateElseCaseChildNotObjectOrAttribute (CreateMethodName, srcfuncname, srcfile...) {
	return <<
		try {
			childobj.xrcObj = objectxrc[child];
			//in special case as bitmapbutton changes 
			// child name to call the correspodent utility
			~(CreateMethodName);
			
			//
			astchild = Utilities [child] (childobj);
			
			if (child == "flag" and astchild.ast==nil)
				continue;
			
			// special case: need to call create function that utilities created
			// e.g: font, not object in xrc but need to produce create function 
			// second step setmethod to setobject2child and continue loop 
			// without add the rest code below
			~(arguments[3]);
			//
			tmpast = <<
						~~(Utilities::lvalue_new(objectname)) [~~methodname] ( ~~(astchild.ast) );
			>>;
			objectast = <<
							~~objectast;
							~~tmpast;
			>>;
			objectsrc += Utilities::tab() + objectname +"."+methodname+" ("+ astchild.src +");"+nl;
		}
		trap e {
			std::error (~(srcfuncname)+" function in "+~(srcfile)+": Not Utilities: " + child);
		}
	>>;
}

function GenerateDecoderFunction (defs) {
	return <<
		function ~(defs.funcname) (object) {
			Utilities::tabincrease();
			local tmpast = nil;
			objectxrc	= object.xrcObj;
			objectid    = ~(defs.type) + ~(defs.id);
			objectname	= objectxrc["$Attributes"].name;
			~(defs.objectname);
			astmap	= object.astMap;
			funcmap	= object.funcMap;
			local childobj = [
				@funcMap : object.funcMap,
				@astMap : astmap,
				@registerWidgetFuncName : object.registerWidgetFuncName
			];
			childobj.parentType = "wx_"+~(defs.type);
			objectsrc = Utilities::tab() + "local " + objectname + " = wx::"+ ~(defs.type) +"_construct();"+nl;
			objectast =
			<<
				local ~~(Utilities::id_new(objectname)) = ~(defs.wxconstruct);
			>>;
			//
			if (object.registerWidgetFuncName) {
				objectast =
				<<
					~~objectast;
					GUIWidgets()[~~objectname] = [ @data : ~~(Utilities::expr_new(objectname)), @type : ~(defs.type) ];
				>>;
			}
			
			foreach (child, std::tabindices(objectxrc)) {
				if (child != "object") {
					if (child == "$Attributes") {
						attributes = objectxrc["$Attributes"];
						attrindices = std::tabindices(attributes);
						foreach (attr, attrindices) {
							if (attr != "class" and attr != "name") {
								local methodname = "set"+attr;
								//
								astattr = Utilities[attr] (attributes);
								//
								tmpast = <<
											~~(Utilities::lvalue_new(objectname)) [~~methodname] ( ~~(astattr.ast) );
								>>;
								objectast = <<
												~~objectast;
												~~tmpast;
								>>;
								objectsrc += Utilities::tab() + objectname+"."+methodname+" ("+ astattr.src +");"+nl;
							}
						}
					}
					else {
						~(defs.elseCaseChildCode);
					}
				}
			}
			
			//
			astguiwindow = <<~(defs.setguiwindow);>>;
			if (Utilities::hasObjectChild(objectxrc) and astguiwindow != nil) {
				objectast = <<
								~~objectast;
								local guichildobj = [];
								~(defs.setguiwindow);
								guichildobj.parent = ~~(Utilities::expr_new(objectname));
				>>;
			}
			else
			if (Utilities::hasObjectChild(objectxrc)) {
				objectast = <<
								~~objectast;
								local guichildobj = [];
								guichildobj.window = guiobj.window;
								guichildobj.parent = ~~(Utilities::expr_new(objectname));
				>>;
			}//TODO: fix case in converter
			else
			if (std::isnil(astguiwindow)) {
				objectast = <<
							~~objectast;
							local guichildobj = [];
							guichildobj.window = guiobj.window;
							//if (guiobj.parent)
								guichildobj.parent = ~~(Utilities::expr_new(objectname));
				>>;
			}
			else {
				objectast = <<
							~~objectast;
							local guichildobj = [];
							~(defs.setguiwindow);
							//if (guiobj.parent)
								guichildobj.parent = ~~(Utilities::expr_new(objectname));
				>>;
			}
			
			
			//
			foreach (child, std::tabindices(objectxrc))
				if (child == "object") {
					~(defs.fixChildObject);
				}
			
			//
			Utilities::tabdecrease();
			//
			createdfuncname = "create_" + ~(defs.type)+"_" + ~(defs.id);
			astmap[objectname]    = [];
			astmap[objectname][0] = createdfuncname;
			astmap[objectname][1] = ~(defs.type);
			
			//
			tmpast = << ~(defs.setobject2parent) >>;
			~(defs.setspecialcasepanel);
			
			bodyast = <<
						~~objectast;
						~(defs.layout);
						~~((object.parentType!=nil?tmpast: nil));
						~(defs.shouldastmethodcalled);
			>>;
			
			~(defs.fixobj2parent);
			
			objectast = <<
							function ~~(Utilities::id_new(createdfuncname)) (~(defs.createfuncargs)) {
								~~bodyast;
								return ~~(Utilities::expr_new(objectname));
							}
			>>;
			objectsrc = Utilities::tab() + "function "+createdfuncname+"() {" + nl + objectsrc + Utilities::tab() + "}";
			createdObject =
			[
				@ast	: objectast,
				@astMap : astmap,
				@funcname : createdfuncname,
				@src : objectsrc
			];
			return createdObject;
		}
	>>;
}



function GenFunctionChoices (defs) {
	return <<
		function ~(defs.funcname) (object) {
			Utilities::tabincrease();
			local type = ~(defs.type);
			local tmpast = nil;
			local createargsast = nil;
			objectxrc	= object.xrcObj;
			objectname	= objectxrc["$Attributes"].name;
			~(defs.objectname);
			//
			id = ~(defs.id);
			funcmap	= object.funcMap;
			astmap	= object.astMap;
			local childobj = [
				@funcMap : object.funcMap,
				@astMap : object.astMap,
				@registerWidgetFuncName : object.registerWidgetFuncName		
			];
			childobj.parentType = "wx_" + ~(defs.type);
			objectsrc = Utilities::tab() + "local " + objectname + " = wx::"+~(defs.type)+"_construct();"+nl;
			objectast =
				<<
					local ~~(Utilities::id_new(objectname)) = ~(defs.wxconstruct);
				>>;
			//
			if (object.registerWidgetFuncName) {
				objectast =
				<<
					~~objectast;
					GUIWidgets()[~~objectname] = [ @data : ~~(Utilities::expr_new(objectname)), @type : ~(defs.type) ];
				>>;
			}
			
			createargs = ~(defs.createargs);
			//
			createobj = ~(defs.createobj);
			//
			
			foreach (child, std::tabindices(createobj)) {
				if (objectxrc[child] != nil) {
					childobj.xrcObj = objectxrc[child];
					astchild = Utilities [child] (childobj);
					//
					if (child == "content") {
						objectsrc += astchild.src + nl;
						objectast = <<
										~~objectast;
										~~(astchild.ast);
						>>;
						createargs [ createobj["content"] ] = <<choices>>;
					}
					else {
						createargs [ createobj[child] ] = astchild.ast;
					}
				}
			}
			for (i=0;i<~(defs.argsize);++i) {
				if (i == 0) {
					createargsast = <<~~(createargs[i])>>;
				}
				else {
					createargsast = <<~~createargsast, ~~(createargs[i])>>;
				}
			}
			objectsrc = Utilities::tab() + objectname + ".create ( " + std::tostring(createargs) + ");" + nl;
			objectast = <<
							~~objectast;
							~~(Utilities::lvalue_new(objectname))["create"] (guiobj.window, ~~(type + id), ~~createargsast);
			>>;
			// set rest elements
			foreach (child, std::tabindices(objectxrc)) {
				if (std::isnil (createobj[child]) and child!="$Attributes") {
					childobj.xrcObj = objectxrc[child];
					//setmethod name
					local methodname = "set";
					if (child == "bg") { methodname += "backgroundcolour"; child = "colour"; }
					else
					if (child == "fg") { methodname += "foregroundcolour"; child = "colour"; }
					else
					if (child == "pos") { methodname = "move"; }
					else
					if (child == "style") { methodname += "window" + child; }
					else
					if (child == "exstyle") { methodname += "extrastyle"; }
					else
					if (child == "hidden") { methodname = "hide"; }
					else
					if (child == "enabled") { methodname = "enable"; }
					else {
						methodname += std::strlower (child);
					}
					
					astchild = Utilities [child] (childobj);
					//
					objectsrc = Utilities::tab() + objectname + "." + methodname +"( " + astchild.src + ");" + nl;
					objectast = <<
								~~objectast;
								~~(Utilities::lvalue_new(objectname))[~~methodname] (~~(astchild.ast));
					>>;
				}
			}
			Utilities::tabdecrease();
			//
			createdfuncname = "create_"+~(defs.type)+"_" + ~(defs.id);
			astmap[objectname]    = [];
			astmap[objectname][0] = createdfuncname;
			astmap[objectname][1] = ~(defs.type);
			//
			objectsrc = Utilities::tab() + "function " + createdfuncname + "( guiobj ) {" + nl + objectsrc + nl + Utilities::tab() + "}" + nl;
			
			objectast = <<
							function ~~(Utilities::id_new(createdfuncname)) (guiobj) {
								~~objectast;
								guiobj["parent"]["setwindow"]( ~~( Utilities::expr_new(objectname) ) );
								return ~~(Utilities::expr_new(objectname));
							}
			>>;
			return [
				@ast	: objectast,
				@astMap	: astmap,
				@funcname : createdfuncname,
				@src : objectsrc
			];
		}
	>>;
}



