const nl = "\n"; 
function GenSizeFunc(name)
{
	return <<function ~(name)(objsize)
		{
			local sizeargs = GenSizeArgs(objsize); 
			astsize = <<wx::size_construct(~~(sizeargs.ast))>>; 
			srcsize = "wx::size_construct(" + sizeargs.src + ")"; 
			return [
				@ast : astsize,
				@src : srcsize
			];
		}>>;
} 
std::inline(GenSizeFunc(<<bitmapsize>>)); 
std::inline(GenSizeFunc(<<size>>)); 
std::inline(GenSizeFunc(<<margins>>)); 
function GenFlagFunc(name)
{
	return <<function ~(name)(objstyle)
		{
			local flagArgs = GenFlagsArgsAst(objstyle); 
			if (flagArgs.ast != nil)
				{
					stylesrc = " wx::flags(" + flagArgs.src + ") "; 
					aststyle = <<wx::flags(~~(flagArgs.ast))>>; 
					return [
						@ast : aststyle,
						@src : stylesrc
					];
				}
			else
				{
					return [
						@src : ""
					];
				}
		}>>;
} 
std::inline(GenFlagFunc(<<style>>)); 
std::inline(GenFlagFunc(<<flag>>)); 
function GenStringFunc(name)
{
	return <<function ~(name)(obj)
		{
			local obj = obj.xrcObj; 
			obj = obj[0]["$CharData"]; 
			if (std::isnil(obj))
				{
					return [
						@ast : <<"">>,
						@src : "\"\""
					];
				} 
			return [
				@ast : <<~~(obj[0])>>,
				@src : obj[0]
			];
		}>>;
} 
std::inline(GenStringFunc(<<title>>)); 
std::inline(GenStringFunc(<<label>>)); 
std::inline(GenStringFunc(<<help>>)); 
std::inline(GenStringFunc(<<longhelp>>)); 
std::inline(GenStringFunc(<<shorthelp>>)); 
std::inline(GenStringFunc(<<tooltip>>)); 
std::inline(GenStringFunc(<<bg>>)); 
std::inline(GenStringFunc(<<fg>>)); 
std::inline(GenStringFunc(<<value>>)); 
std::inline(GenStringFunc(<<filter>>)); 
std::inline(GenStringFunc(<<path>>)); 
function GenNumberFunc(name)
{
	return <<function ~(name)(obj)
		{
			local obj = obj.xrcObj; 
			obj = obj[0]["$CharData"]; 
			return [
				@ast : <<~~(std::strtonum(obj[0]))>>,
				@src : obj[0]
			];
		}>>;
} 
std::inline(GenNumberFunc(<<separation>>)); 
std::inline(GenNumberFunc(<<packing>>)); 
std::inline(GenNumberFunc(<<fields>>)); 
std::inline(GenNumberFunc(<<centered>>)); 
std::inline(GenNumberFunc(<<border>>)); 
std::inline(GenNumberFunc(<<option>>)); 
std::inline(GenNumberFunc(<<vgap>>)); 
std::inline(GenNumberFunc(<<hgap>>)); 
std::inline(GenNumberFunc(<<rows>>)); 
std::inline(GenNumberFunc(<<cols>>)); 
std::inline(GenNumberFunc(<<growablecols>>)); 
std::inline(GenNumberFunc(<<growablerows>>)); 
std::inline(GenNumberFunc(<<maxlength>>)); 
std::inline(GenNumberFunc(<<dimension>>)); 
std::inline(GenNumberFunc(<<selection>>)); 
std::inline(GenNumberFunc(<<font_size>>)); 
std::inline(GenNumberFunc(<<selected>>)); 
std::inline(GenNumberFunc(<<index>>)); 
std::inline(GenNumberFunc(<<numval>>)); 
std::inline(GenNumberFunc(<<wrap>>)); 
function GenBoolFunc(name)
{
	return <<function ~(name)(obj)
		{
			local obj = obj.xrcObj; 
			obj = obj[0]["$CharData"]; 
			return [
				@ast : <<~~((std::strtonum(obj[0]) != 0))>>,
				@src : ( std::strtonum(obj[0]) != 0 ? "true" : "false" )
			];
		}>>;
} 
std::inline(GenBoolFunc(<<font_underlined>>)); 
std::inline(GenBoolFunc(<<enabled>>)); 
std::inline(GenBoolFunc(<<checked>>)); 
std::inline(GenBoolFunc(<<boolval>>)); 
function GenAttr(def)
{
	return <<function ~(def.funcname)(attributes)
		{
			return [
				@ast : <<~~(attributes[~(def.index)])>>,
				@src : "\"" + attributes[~(def.index)] + "\""
			];
		}>>;
} 
std::inline(GenAttr([
	@funcname : <<name>>,
	@index : "name"
])); 
std::inline(GenAttr([
	@funcname : <<id>>,
	@index : "id"
])); 
function default(defaultobj)
{
	return [
		@src : ""
	];
} 
function hidden(hiddenobj)
{
	return [
		@src : ""
	];
} 
function GenFontStringFunc(name)
{
	return <<function ~(name)(obj)
		{
			local obj = obj.xrcObj; 
			obj = obj[0]["$CharData"]; 
			if (std::isnil(obj))
				{
					return [
						@ast : <<"">>,
						@src : "\"\""
					];
				} 
			return [
				@ast : <<~~(std::strupper(obj[0]))>>,
				@src : std::strupper(obj[0])
			];
		}>>;
} 
std::inline(GenFontStringFunc(<<font_family>>)); 
std::inline(GenFontStringFunc(<<font_style>>)); 
std::inline(GenFontStringFunc(<<font_weight>>)); 
std::inline(GenStringFunc(<<font_face>>)); 
std::inline(GenStringFunc(<<colourcode>>)); 
std::inline(GenNumberFunc(<<sashpos>>)); 
std::inline(GenNumberFunc(<<gravity>>)); 
std::inline(GenNumberFunc(<<minsize>>));