function CheckValidName(name) {
	if (std::isnil(name))
		std::error ("Not name at attribute in model!");
	else if(not std::strisident(name))
		std::error ("Name '" + name + "' is not a valid id.");	
}

function id_new(name) { return std::ast_new("Name", [@name : name]); }

function lvalue_new(name) {
	local lvalue = std::ast_new("LvalueIdent");
	lvalue.push_back(id_new(name), "name");
	return lvalue;
}

function expr_new(name) {
	local lvalue = lvalue_new(name);
	local primaryExpr = std::ast_new("PrimaryExpression"); 
	primaryExpr.push_back(lvalue, "expr"); 
	return primaryExpr;
}

function GetDeltaType (type) {
	local deltaType = nil;
	if (type != nil) {
		type = std::strtokenise(type, "#//").back();
		// bool Delta type
		if (type == "EBoolean")
			deltaType = std::TYPEOF_BOOL;
		// string Delta type
		else if (type == "EString" or type == "ECharacterObject")
			deltaType = std::TYPEOF_STRING;
		// number Delta type
		else if ( type == "EBigDecimal"		or
				  type == "EBigInteger"		or
				  type == "EInt"			or
				  type == "EIntegerObject"	or
				  type == "ELong"			or
				  type == "ELongObject"		or
				  type == "EFloat"			or
				  type == "EFloatObject" )
			deltaType = std::TYPEOF_NUMBER;
	}
	return deltaType;
}

function GetDeltaTypeAst (type) {
	local deltaType = nil;
	if (type != nil) {
		type = std::strtokenise(type, "#//").back();
		// bool Delta type
		if (type == "EBoolean")
			deltaType = <<std::TYPEOF_BOOL>>;
		// string Delta type
		else if (type == "EString" or type == "ECharacterObject")
			deltaType = <<std::TYPEOF_STRING>>;
		// number Delta type
		else if ( type == "EBigDecimal"		or
				  type == "EBigInteger"		or
				  type == "EInt"			or
				  type == "EIntegerObject"	or
				  type == "ELong"			or
				  type == "ELongObject"		or
				  type == "EFloat"			or
				  type == "EFloatObject" )
			deltaType = <<std::TYPEOF_NUMBER>>;
	}
	return deltaType;
}

function GetValueByType (value, type) {
	local valuebt = nil;
	if (type != nil) {
		type = std::strtokenise(type, "#//").back();
		
		// bool Delta type
		if (type == "EBoolean")
			valuebt = (value=="true" ? true : false);
		// string Delta type
		else if (type == "EString" or type == "ECharacterObject")
			valuebt = std::tostring((value?value:""));
		// number Delta type
		else if ( type == "EBigDecimal"		or
				  type == "EBigInteger"		or
				  type == "EInt"			or
				  type == "EIntegerObject"	or
				  type == "ELong"			or
				  type == "ELongObject"		or
				  type == "EFloat"			or
				  type == "EFloatObject" )
			valuebt = (value ? std::strtonum(value) : 0);
		// not supported type from Delta
		else
			valuebt = nil;
	}
	else if (value != nil)
		valuebt = value;
	//else
	//	std::error ("Error: no type or value in eAttribute model!");
	//
	return valuebt;
}
