

const nl = "\n";
numtabs = 0;

function tabincrease () { ++::numtabs; }
function tabdecrease () { --::numtabs; }

function tab () {
	srctab = "";
	for (i=0; i<numtabs; ++i) { srctab += "\t"; }
	return srctab;
}

function hasObjectChild (objectxrc) {
	foreach (child, std::tabindices(objectxrc)) {
		if (child == "object" or child == "font") {
			return true;
		}
	}
	return false;
}


function id_new(name) { return std::ast_new("Name", [@name : name]); }

function lvalue_new(name) {
	local lvalue = std::ast_new("LvalueIdent");
	lvalue.push_back(id_new(name), "name");
	return lvalue;
}
function expr_new(name) {
	local lvalue = lvalue_new(name);
	local primaryExpr = std::ast_new("PrimaryExpression"); 
	primaryExpr.push_back(lvalue, "expr"); 
	return primaryExpr;
}

/**
 * Returns object : [@width:number1, @height:number2]
 *
 **/
 
function GenSizeArgs (objsize) {
	local objsize = objsize.xrcObj;
	try {
		objsize = objsize[0]["$CharData"];
	}
	trap e {
		return [ @ast : <<-1, -1>>, @src : "-1, -1"];
	}
	if (std::isnil(objsize) or std::isnil(objsize[0])) {
		return [ @ast : <<-1, -1>>, @src : "-1, -1"];
	}
	list = std::strtokenise(objsize[0],",");
	//
	if (list.total()!=2) {
		std::error("Function=\"size_decoder\", the list elements invalid!");
	}
	width	= std::strtonum(list.front());
	height	= std::strtonum(list.back());
	astsize = << ~width, ~height >>;
	srcsize = width + ", " + height;
	return [ @ast : astsize, @src : srcsize ];
}

&function GenSizeFunc(name) {
	 return <<
		function ~(name) (objsize) {
			local sizeargs = GenSizeArgs (objsize);
			astsize = << wx::size_construct (~~(sizeargs.ast)) >>;
			srcsize = "wx::size_construct(" + sizeargs.src + ")";
			return [ @ast : astsize, @src : srcsize ];
		}
	>>;
}

! (GenSizeFunc (<<bitmapsize>>));
! (GenSizeFunc (<<size>>));
! (GenSizeFunc (<<margins>>));


function cellspan (objgbspan) {
	local sizeargs = GenSizeArgs (objgbspan);
	astsize = << wx::gbspan_construct (~(sizeargs.ast)) >>;
	srcsize = "wx::gbspan_construct(" + sizeargs.src + ")";
	return [ @ast : astsize, @src : srcsize ];
}

function cellpos (objgbpos) {
	local sizeargs = GenSizeArgs (objgbpos);
	astsize = << wx::gbposition_construct (~(sizeargs.ast)) >>;
	srcsize = "wx::gbspan_construct(" + sizeargs.src + ")";
	return [ @ast : astsize, @src : srcsize ];
}

function pos (objpos) {
	local sizeargs = GenSizeArgs (objpos);
	astsize = << wx::point_construct (~(sizeargs.ast)) >>;
	srcsize = "wx::point_construct(" + sizeargs.src + ")";
	return [ @ast : astsize, @src : srcsize ];
}

function GenFlagsArgsAst (objstyle) {
	local objstyle = objstyle.xrcObj;
	objstyle = objstyle[0]["$CharData"];
	local astflags = nil;
	index = 0;
	if (std::isnil(objstyle) or std::isnil(objstyle[0])) {
		return [ /*@ast : nil,*/ @src : " " ];
	}
	list = std::strtokenise(objstyle[0]," |");
	// delta wxwidgets constants starts without the "wx" need transform
	astyle = std::strtokenise (list.front(), "wx").back();
	astflags = <<~(astyle)>>;
	stylesrc = astyle;
	//
	first = true;
	foreach (wxstyle, list) {
		if (first == false) {
			// delta wxwidgets constants starts without the "wx" need transform
			astyle = std::strtokenise(wxstyle, "wx").back();
			stylesrc += ", " + astyle;
			astflags = << ~astflags, ~(/*id_new("wx::"+*/astyle/*)*/) >>;
		}
		else {
			//first style is already in ast before loop
			first = false;
		}
	}
	return [ @ast : astflags, @src : stylesrc ];
}

&function GenFlagFunc (name) {
	return <<
		function ~(name) (objstyle) {
			local flagArgs = GenFlagsArgsAst (objstyle);
			if (flagArgs.ast != nil) {
				stylesrc = " wx::flags(" + flagArgs.src + ") ";
				aststyle = <<wx::flags(~~(flagArgs.ast))>>;
				return [ @ast : aststyle, @src : stylesrc ];
			}
			else {
				return [ @src : "" ];
			}
		}
	>>;
}

! (GenFlagFunc(<<style>>));
! (GenFlagFunc(<<flag>>));

/**
 * Returns title's string or an empty string if title doesn't exist
 *
 **/

&function GenStringFunc(name) {
	 return <<
		function ~(name) (obj) {
			local obj = obj.xrcObj;
			obj = obj[0]["$CharData"];
			if (std::isnil(obj)) {
				return [ @ast : <<"">>, @src : "\"\"" ];
			}
			return [ @ast : <<~~(obj[0])>>, @src : obj[0] ];
		}
	>>;
}

!(GenStringFunc( <<title>>		));
!(GenStringFunc( <<label>>		));
!(GenStringFunc( <<help>>		));
!(GenStringFunc( <<longhelp>>	));
!(GenStringFunc( <<shorthelp>>	));
!(GenStringFunc( <<tooltip>>	));
!(GenStringFunc( <<bg>>			));
!(GenStringFunc( <<fg>>			));
!(GenStringFunc( <<value>>		));
!(GenStringFunc( <<filter>>		));
!(GenStringFunc( <<path>>		));


&function GenNumberFunc (name) {
	return <<
		function ~(name) (obj) {
			local obj = obj.xrcObj;
			obj = obj[0]["$CharData"];
			return [ @ast : <<~~(std::strtonum(obj[0]))>>, @src : obj[0] ];
		}
	>>;
}

! (GenNumberFunc ( <<separation>>	));
! (GenNumberFunc ( <<packing>>		));
! (GenNumberFunc ( <<fields>>		));
! (GenNumberFunc ( <<centered>>		));
! (GenNumberFunc ( <<border>>		));
! (GenNumberFunc ( <<option>>		));
! (GenNumberFunc ( <<vgap>>			));
! (GenNumberFunc ( <<hgap>>			));
! (GenNumberFunc ( <<rows>>			));
! (GenNumberFunc ( <<cols>>			));
! (GenNumberFunc ( <<growablecols>>	));
! (GenNumberFunc ( <<growablerows>>	));
! (GenNumberFunc ( <<maxlength>>	));
! (GenNumberFunc ( <<dimension>>	));
! (GenNumberFunc ( <<selection>>	));
! (GenNumberFunc ( <<font_size>>	));
! (GenNumberFunc ( <<selected>>		));
! (GenNumberFunc ( <<index>>		));
! (GenNumberFunc ( <<numval>>		));
! (GenNumberFunc ( <<wrap>>			));


&function GenBoolFunc (name) {
	return <<
		function ~(name) (obj) {
			local obj = obj.xrcObj;
			obj = obj[0]["$CharData"];
			return [ @ast : <<~~((std::strtonum(obj[0])!=0))>>, @src : (std::strtonum(obj[0])!=0?"true":"false") ];
		}
	>>;
}

! (GenBoolFunc ( <<font_underlined>>	));
! (GenBoolFunc ( <<enabled>>			));
! (GenBoolFunc ( <<checked>>			));
! (GenBoolFunc ( <<boolval>>			));

function bitmap (bitmapobj) {
	local bitmapobj = bitmapobj.xrcObj;
	bitmapobj = bitmapobj[0]["$CharData"];
	if (std::isnil(bitmapobj)) {
		return [ @ast : <<wx::bitmap_construct()>>, @src : "wx::bitmap_construct()" ];
	}
	return [ @ast : <<wx::bitmap_construct(wx::image_construct(~(bitmapobj[0])))>>, @src : "wx::bitmap_construct("+bitmapobj[0]+")" ];
}

function orient (objectchildsobj) {	
	orientobj = objectchildsobj.xrcObj;
	orientobj = orientobj[0]["$CharData"];
	orientsrc = std::strtokenise(orientobj[0],"wx").back();
	return [ @ast : <<~orientsrc>>, @src : orientsrc ];
}

&function GenAttr (def) {
	return <<
		function ~(def.funcname) (attributes) {
			return [
				@ast : <<~~(attributes[~(def.index)])>>,
				@src : "\""+attributes[~(def.index)]+"\""
			];
		}
	>>;
}

! (GenAttr ( [@funcname : <<name>>, @index : "name"] ));
! (GenAttr ( [@funcname : <<id>>, @index : "id"] ));


function default (defaultobj) 	{ return [ @src : "" ]; }
function hidden  (hiddenobj)	{ return [ @src : "" ]; }

/******************** font ************************************************/


&function GenFontStringFunc(name) {
	 return <<
		function ~(name) (obj) {
			local obj = obj.xrcObj;
			obj = obj[0]["$CharData"];
			if (std::isnil(obj)) {
				return [ @ast : <<"">>, @src : "\"\"" ];
			}
			return [ @ast : <<~~(std::strupper(obj[0]))>>, @src : std::strupper(obj[0]) ];
		}
	>>;
}

! (GenFontStringFunc (<<font_family>>));
! (GenFontStringFunc (<<font_style>>));
! (GenFontStringFunc (<<font_weight>>));
//
! (GenStringFunc (<<font_face>>));


fontid = 0;

function font (fontobj) {
	::tabincrease();
	local childarg = [
		@funcMap : fontobj.funcMap,
		@astMap : fontobj.astMap,
		@registerWidgetFuncName : fontobj.registerWidgetFuncName
	];
	local fontobj = (fontobj.xrcObj)[0];
	++::fontid;
	local fontast = <<local font = wx::font_construct();>>;
	local fontsrc = ::tab() + "local font = wx::font_construct();" + nl;
	foreach (child, std::tabindices(fontobj)) {
		childarg.xrcObj = fontobj[child];
		try {
			childfunc = "font_"+child;
			childobj = std::callglobal (childfunc, childarg);
			// fix method name
			local methodname = "set";
			if (child == "face") {
				methodname += "facename";
			}
			else
			if (child == "size") {
				methodname += "pointsize";
			}
			else {
				methodname += child;
			}
			//
			fontsrc += ::tab() + "font." + methodname + "( " + childobj.src + " );" + nl;
			fontast = <<
						~fontast;
						font[ ~methodname ] ( ~(childobj.ast) );
			>>;
		}
		trap e {
			std::error ("Error: Not supported in wxFont invalid xrc child named as " + child + "!\n");
		}
	}
	::tabdecrease();
	local createdfuncname = "create_font"+::fontid;
	
	fontsrc = ::tab() + "function " + createdfuncname +" ( guiobj ) {" + nl + fontsrc + nl + ::tab() + "}" + nl;
	fontast = <<
				function ~(::id_new(createdfuncname)) (guiobj) {
					~fontast;
					return font;
				}
	>>;
	return [ @funcname : createdfuncname, @ast : fontast, @src : fontsrc ];
}


/******************** colour *************************************************/

! (GenStringFunc (<<colourcode>>));


function colour (colourobj) {
	ccolour = colourcode (colourobj);
	return [ @ast : <<wx::colour_construct(~(ccolour.ast))>>, @src : "wx::colour_construct( " + ccolour.src + " )" ];
}


/******************** content ************************************************/

function conitem (item) {
	local item = item.xrcObj;
	item = item["$CharData"][0];
	if (std::isnil(item)) {
		return [
			@ast : <<"">>,
			@src : "\"\""
		];
	}
	return [
		@ast : <<~(item)>>,
		@src : item
	];
}

contentid = 0;

function content (contentobj) {
	local astmap = contentobj.astMap;
	local funcmap = contentobj.funcMap;
	local childarg = [
		@funcMap : funcmap,
		@astMap : astmap,
		@registerWidgetFuncName : contentobj.registerWidgetFuncName
	];
	local contentobj = (contentobj.xrcObj)[0]["item"];
	
	++::contentid;
	local contentast = nil;
	local contentsrc = ::tab() + "local choices = [ ";
	isfirstchild = true;
//	foreach (child, std::tabindices(contentobj)) {
	for (i=0;i<std::tablength(contentobj);++i) {
		childarg.xrcObj = contentobj[i];
		try {
			childobj = ::conitem(childarg);
			//
			if (isfirstchild) {
				isfirstchild = false;
				contentsrc += childobj.src;
				contentast = << ~(childobj.ast) >>;
			}
			else {
				contentsrc += ", " + childobj.src;
				contentast = << ~contentast, ~(childobj.ast) >>;
			}
		}
		trap e {
			std::error ("Error: Not supported in wxFont invalid xrc child 'item' named as " + i + "!\n");
		}
	}
	contentsrc += "];"+nl;
	contentast = <<
				local choices = [ ~contentast ];
	>>;
	return [
		@ast : contentast,
		@astMap	: astmap,
		@funcname : createdfuncname,
		@src : contentsrc
	];
}


/////////////////////  splitter window utilites  ////////////////////////////////



! (GenNumberFunc ( <<sashpos>>	));
! (GenNumberFunc ( <<gravity>>	));
! (GenNumberFunc ( <<minsize>>	));


function splitmode (splitmodeobj) {
	local splitmodeobj = splitmodeobj.xrcObj;
	splitmodeobj = splitmodeobj[0]["$CharData"];
	if (std::isnil(splitmodeobj)) {
		std::error ("Error: Invalid XRC input file.\nSplitmode(orientation for wxFormBuider) in SplitterWindow has no value!");
	}
	return [ @ast : <<~(((splitmodeobj[0]=="horizontal")?"SPLIT_HORIZONTAL":"SPLIT_VERTICAL"))>>, @src : (splitmodeobj[0]=="horizontal"?"SPLIT_HORIZONTAL":"SPLIT_VERTICAL") ];
}

